
; Variable Macro Mouse v1.0
; Undetectable rapid fire input due to companies who won't make responsible design decisions.
;
pullup on
input C.3
output C.2
symbol BUTTON2 = pinC.3
symbol HOLD = b0
symbol RELEASE = b1
symbol VHOLD = b2
symbol VRELEASE = b3
symbol FATIGUE = b4
symbol MFATIGUE = b5
symbol FVHOLD = b6
symbol FVRELEASE = b7
symbol MRELEASE = b8
symbol MHOLD = w5
symbol HRAND = w6
symbol RRAND = w7

; Taking Nikke as an example, ideal rate is achieved with 200ms hold, 15ms release
; If you run with those settings, you will probably get warned and kicked off the game, so be more reasonable
; about settings which are human-like, for example the below values are based on my own recorded inputs.
let HOLD = 200
let RELEASE = 35

; Variance windows, remember these are ADDED to the above, again based on personal recordings.
let VHOLD = 10
let VRELEASE = 7

; Pick ONE of the following initial C.2 states
; If mouse left button is active high (one pin is connected to 5v), uncomment the following line:
;low C.2
; If mouse left button is active low (one pin is connected to ground), uncomment the following line:
high C.2

; Explanation of calculations in the run loop:
; MFATIGUE increases due to simulated player FATIGUE
; FVHOLD / FVRELEASE are the MFATIGUE increased variance windows
; MHOLD/MRELEASE are random numbers within the variance windows, note that with picaxe MCU the random function
; generates 0-65535 hence using the // operator to then force this value within the variance range.
; For example, random 0-10 would be (random result)//10
; MHOLD/MRELEASE are then increased by the baseline HOLD and RELEASE values
; The strange approach is caused by strict left to right arithmetic in picaxe chips.

do
  ; Fatigue resets as soon as the button is released to simulate rest
  let FATIGUE = 0
  let MFATIGUE = 0
  
  do while BUTTON2 = 0 ; NOTE - set to 1 for active high right button (one pin connected to 5v)
	  
	  random HRAND
	  random RRAND
	  let FVHOLD = VHOLD + MFATIGUE
	  let FVRELEASE = VRELEASE + MFATIGUE
	  let MHOLD = HRAND // FVHOLD
	  let MRELEASE = RRAND // FVRELEASE
	  let MHOLD = MHOLD + HOLD
	  let MRELEASE = MRELEASE + RELEASE
	  
	  ; Leave this commented out for actual use as debug output slows execution significantly
	  ;sertxd("Hold: ",#MHOLD,"   Release: ",#MRELEASE,"   Fatigue adj: +",#MFATIGUE,13,10)
	  
	  ; Use ONE of the following C.2 (left button) toggles
	  ; If mouse left button is active high (one pin is connected to 5v), uncomment the following line:
	  ;high C.2
	  ; If mouse left button is active low (one pin is connected to ground), uncomment the following line:
	  low C.2
	  
	  pause MHOLD
	  
	  ; Use ONE of the following C.2 (left button) toggles
	  ; If mouse left button is active high (one pin is connected to 5v), uncomment the following line:
	  ;low C.2
	  ; If mouse left button is active low (one pin is connected to ground), uncomment the following line:
	  high C.2
	  
	  pause MRELEASE
	  
	  ; Fatigue simulation - fatigue adjust is a 5th of FATIGUE, causing slow ramp up of variance
	  ; Comment out both of the lines below if you want to totally disable fatigue emulation
	  if FATIGUE < 75 then let FATIGUE = FATIGUE + 1 endif
	  let MFATIGUE = FATIGUE / 5
  loop
loop

